package classcontrol.controller;

import classcontrol.dao.ConstantsForm;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;

/**
 * Handles all Class Control business logic with dates.
 *
 * @author Cristian TG
 * @since 0.1-ALPHA
 * @version 0.2-BETA
 */
public final class DatesUtil {

    /**
     * Start year delimiter.
     */
    private final static char START_YEAR = '{';
    /**
     * End year delimiter.
     */
    private final static char END_YEAR = '}';

    /**
     * Calculates all saturdays and Sundays between two dates (both included).
     * INPUT: Two dates parsed with:
     * {@link ConstantsForm#DATE_SIMPLE_DATE_FORMAT}
     *
     * OUTPUT:
     * 2013{07-1208-1214-1215-1221-1222-1228-1229-12}2014{04-0105-0111-0112-0118-0119-0125-0126-01}
     *
     * @param startDate Initial date (included) with "INPUT" format.
     * @param endDate Final date (included) with "INPUT" format.
     * @return Saturdays and Sundays in "OUTPUT" format or null if
     * startDate.after(endDate) or null if there are not any Saturdays and
     * Sundays between dates.
     */
    public static String getSaturdaySundaysBetweenTwoDates(final Date startDate, final Date endDate) {
        if (startDate.after(endDate)) {
            return null;
        }
        // 1. Get dates
        final Calendar startCal = Calendar.getInstance();
        startCal.setTime(startDate);
        final Calendar endCal = Calendar.getInstance();
        endCal.setTime(endDate);

        // 2. Get years
        final int startYear = startCal.get(Calendar.YEAR);
        int currentYear = startYear;
        int current = startCal.get(Calendar.DAY_OF_WEEK);
        final StringBuilder currentDates = new StringBuilder("").append(currentYear).append(START_YEAR);
        final SimpleDateFormat formatter = new SimpleDateFormat("dd" + ConstantsForm.DAY_MONTH_SEPARATOR + "MM");
        if (current == Calendar.SATURDAY || current == Calendar.SUNDAY) {
            currentDates.append(formatter.format(startCal.getTime()));
        }

        // 3. Saturdays and Sundays
        boolean oneAtLeast = false;
        final long endCalMillis = endCal.getTimeInMillis();
        while (startCal.getTimeInMillis() < endCalMillis) {
            //excluding start date (Cristian  TG, but before was included)
            startCal.add(Calendar.DAY_OF_MONTH, 1);
            current = startCal.get(Calendar.DAY_OF_WEEK);
            if (current == Calendar.SATURDAY || current == Calendar.SUNDAY) {
                if (currentYear != startCal.get(Calendar.YEAR)) {
                    //1. Close previous date
                    currentDates.append(END_YEAR);
                    currentYear++;
                    //2. Start new date
                    currentDates.append(currentYear).append(START_YEAR);
                }
                currentDates.append(formatter.format(startCal.getTime()));
                if (!oneAtLeast) {
                    oneAtLeast = true;
                }
            }
        } //excluding end date
        return !oneAtLeast ? null : currentDates.append(END_YEAR).toString();
    }

    /**
     *
     * INPUT:
     * 2013{07-1208-1214-1215-1221-1222-1228-1229-12}2014{04-0105-0111-0112-0118-0119-0125-0126-01}
     * OUTPUT:
     * 07-12-2013,08-12-2013,14-12-2013,15-12-2013,21-12-2013,22-12-2013,28-12-2013,29-12-2013,04-01-2014,05-01-2014,11-01-2014,12-01-2014,18-01-2014,19-01-2014,25-01-2014,26-01-2014
     *
     * @param data Dates with "INPUT" format.
     * @return Dates with "OUTPUT" format or an empty String if there are not
     * dates.
     */
    public static StringBuilder getDatepickerDates(final String data) {
        final StringBuilder formattedString = new StringBuilder("");
        if (data == null) {
            return formattedString;
        }
        final int SIZE = data.length();
        final String YEAR = "yyyy";
        final String MINIUM = YEAR + "{dd-MM}";
        final int YEAR_SIZE_PLUS_1 = YEAR.length() + 1;
        if (SIZE >= MINIUM.length()) {
            int startYearData = data.indexOf("{");
            final String firstYear = data.substring(0, startYearData);
            int currentYear = Integer.parseInt(firstYear);

            char currentChar;
            byte currentDay = 0;
            boolean firstTime = true;
            boolean isFirst = false;
            int previousYear = currentYear;
            final char DAY_MONTH_SEPARATOR = ConstantsForm.DAY_MONTH_SEPARATOR;
            for (int i = startYearData + 1; i < SIZE; i++) {
                currentChar = data.charAt(i);
                if (((currentDay == 0 || currentDay == 1 || currentDay == 3 || currentDay == 4) && !(currentChar >= '0' && currentChar <= '9'))
                        || (currentDay == 2 && currentChar != ConstantsForm.DAY_MONTH_SEPARATOR) || ((currentDay == 5) && !(currentChar == END_YEAR || (currentChar >= '0' && currentChar <= '9')))) {
                    formattedString.delete(0, formattedString.length());
                    return formattedString;
                }
                if (currentChar != END_YEAR) {
                    if (currentDay == 5) {
                        formattedString.append(DAY_MONTH_SEPARATOR).append(isFirst ? previousYear : currentYear)
                                .append(ConstantsForm.MULTI_DATE_SEPARATOR);
                        if (firstTime) {
                            firstTime = false;
                        }
                        currentDay = 0;
                        isFirst = false;
                    }
                    formattedString.append(currentChar);
                    currentDay++;
                } else {
                    try {
                        previousYear = currentYear;
                        currentYear = Integer.parseInt(data.substring(i + 1, i + YEAR_SIZE_PLUS_1));

                    } catch (final Exception ex) {
                    }
                    i = i + YEAR_SIZE_PLUS_1;
                    isFirst = true;
                }
            }
            formattedString.append(DAY_MONTH_SEPARATOR).append(currentYear);
        }
        return formattedString;
    }

    /**
     * INPUT:
     * 2013{07-1208-1214-1215-1221-1222-1228-1229-12}2014{04-0105-0111-0112-0118-0119-0125-0126-01}
     * OUTPUT:
     * '07-12-2013','08-12-2013','14-12-2013','15-12-2013','21-12-2013','22-12-2013','28-12-2013','29-12-2013','04-01-2014','05-01-2014','11-01-2014','12-01-2014','18-01-2014','19-01-2014','25-01-2014','26-01-2014'
     *
     * @param data
     * @return Empty String if there are not dates.
     */
    public static StringBuilder getFormattedDatepickerDates(final String data) {
        if (data == null) {
            return new StringBuilder("");
        }
        final StringBuilder myData = getDatepickerDates(data);
        final int DATA_SIZE = myData.length();
        final int MINIUM_SIZE = "dd-MM-yyyy".length();
        final String FORMAT = "'";
        final String MULTIDATE_SEPARATOR = "" + ConstantsForm.MULTI_DATE_SEPARATOR;

        return DATA_SIZE < MINIUM_SIZE ? myData : new StringBuilder(FORMAT).append(myData.toString().replaceAll(MULTIDATE_SEPARATOR, "','")).append(FORMAT);
    }

    /**
     * INPUT:
     * 07-12-2013,08-12-2013,14-12-2013,15-12-2013,21-12-2013,22-12-2013,28-12-2013,29-12-2013,04-01-2014,05-01-2014,11-01-2014,12-01-2014,18-01-2014,19-01-2014,25-01-2014,26-01-2014
     * IN ANY ORDER
     *
     * @param data Dates with "INPUT" format.
     * @return A non-repeated and sorted dates list.
     */
    public static ArrayList<Date> getDatesFromDatePicker(final String data) {
        final ArrayList<Date> myDates = new ArrayList<Date>(5);
        int previousIndex = 0;
        int currentIndex = data.indexOf("" + ConstantsForm.MULTI_DATE_SEPARATOR);

        try {
            while (currentIndex != -1) {
                myDates.add(ConstantsForm.DATE_SIMPLE_DATE_FORMAT.parse(data.substring(previousIndex, currentIndex)));
                previousIndex = currentIndex + 1;
                currentIndex = data.indexOf("" + ConstantsForm.MULTI_DATE_SEPARATOR, previousIndex);
            }
            // Last date:
            myDates.add(ConstantsForm.DATE_SIMPLE_DATE_FORMAT.parse(data.substring(previousIndex, data.length())));
        } catch (final ParseException ex) {
            //ex.printStackTrace();
            return myDates;
        }
        // 2. Order dates
        if (myDates.isEmpty()) {
            return myDates;
        }
        // Delete duplicates
        final HashSet hs = new HashSet();
        hs.addAll(myDates);
        myDates.clear();
        myDates.addAll(hs);
        // Sort
        Collections.sort(myDates);
        return myDates;
    }

    /**
     *
     * INPUT:
     * 07-12-2013,08-12-2013,14-12-2013,15-12-2013,21-12-2013,22-12-2013,28-12-2013,29-12-2013,04-01-2014,05-01-2014,11-01-2014,12-01-2014,18-01-2014,19-01-2014,25-01-2014,26-01-2014
     * IN ANY ORDER
     *
     * OUTPUT:
     * 2013{07-1208-1214-1215-1221-1222-1228-1229-12}2014{04-0105-0111-0112-0118-0119-0125-0126-01}
     *
     * @param data Dates with "INPUT" format.
     * @return Dates with "OUTPUT" format (ordered minor to mayor). If date has
     * not valid format, it returns an empty string. If there are any repeat
     * date, output only one.
     */
    public static String getDatabaseDates(final String data) {
        if (data == null || data.length() == 0) {
            return "";
        }
        // 1. Parse data:
        final ArrayList<Date> myDates = getDatesFromDatePicker(data);
        final StringBuilder parsedString = new StringBuilder("");
        for (final Date myDate : myDates) {
            parsedString.append(ConstantsForm.DATE_SIMPLE_DATE_FORMAT.format(myDate)).append(ConstantsForm.MULTI_DATE_SEPARATOR);
        }
        // Remove last char
        if (!myDates.isEmpty()) {
            parsedString.deleteCharAt(parsedString.length() - 1);
        }

        // 3. Get output
        final String parsedData = parsedString.toString();
        final int SIZE = parsedData.length();
        final StringBuilder formattedString = new StringBuilder("");
        // 1. Total years
        final char DAY_MONTH_SEPARATOR = ConstantsForm.DAY_MONTH_SEPARATOR;
        final int firstDayMonthSeparator = parsedData.indexOf(DAY_MONTH_SEPARATOR) + 1;
        final int secondDayMonthSeparator = parsedData.indexOf(DAY_MONTH_SEPARATOR, firstDayMonthSeparator) + 1;
        String currentYear = parsedData.substring(secondDayMonthSeparator, secondDayMonthSeparator + 4);
        String auxYear;
        String auxData;
        int auxSize;
        final byte SUSTRACTION = 5;
        byte currentChar = 0;
        char charValue;
        formattedString.append(currentYear).append(START_YEAR);
        final char MULTI_DATE_SEPARATOR = ConstantsForm.MULTI_DATE_SEPARATOR;
        for (int i = 0; i < SIZE; i++) {

            charValue = parsedData.charAt(i);
            if (((currentChar == 0 || currentChar == 1 || currentChar == 3 || currentChar == 4) && (charValue < '0' && charValue > '9')) || ((currentChar == 2 || currentChar == 5) && (charValue != DAY_MONTH_SEPARATOR)) || (currentChar == 10 && charValue != MULTI_DATE_SEPARATOR)) {
                return "";
            }
            if (currentChar < 5) {
                formattedString.append(charValue);
            } else {
                if (currentChar == 6) {
                    // Get year
                    auxYear = parsedData.substring(i, i + 4);
                    if (auxYear.compareTo(currentYear) != 0) {
                        currentYear = auxYear;
                        auxSize = formattedString.length();
                        auxData = formattedString.substring(auxSize - SUSTRACTION, auxSize);
                        formattedString.delete(auxSize - SUSTRACTION, auxSize);
                        formattedString.append(END_YEAR).append(currentYear).append(START_YEAR).append(auxData);
                    }
                } else {
                    if (currentChar == 10) {
                        currentChar = -1;//-1+1=0
                    }
                }
            }
            currentChar++;
        }
        return formattedString.append(END_YEAR).toString();
    }
}