package classcontrol.dao;

import classcontrol.entity.EvaluationPeriod;
import classcontrol.entity.SubjectCompetence;
import classcontrol.utils.Constants;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

/**
 * Handles all abasences database connections.
 *
 * @author Cristian TG
 * @since 0.1-ALPHA
 * @version 0.2-BETA
 */
final class SubjectCompetenceDAO {

    /**
     * Returns all teacher's subject competences in a list alphabetically
     * ordered.
     *
     * @param idEdmodoTeacher Edmodo teacher id.
     * @return all teacher's subject competences in a list alphabetically
     * ordered.
     * @throws IllegalArgumentException If any argument is invalid.
     */
    static List<SubjectCompetence> getSubjectsCompetences(final String idEdmodoTeacher) throws IllegalArgumentException {
        if (idEdmodoTeacher == null) {
            throw new IllegalArgumentException("No valid Edmodo id.");
        }
        final List<SubjectCompetence> mySubjectsCompetencesList = new ArrayList<SubjectCompetence>();
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            ps = con.prepareStatement(new StringBuilder("SELECT * FROM ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE).append(',')
                    .append(ConstantsDB.VIEW_TEACHER)
                    .append(" WHERE ").append(ConstantsDB.TABLE_TEACHER_ATT_ID_EDMODO).append("=? AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append("=").append(ConstantsDB.TABLE_TEACHER_ATT_ID_TEACHER).toString());
            ps.setString(1, idEdmodoTeacher);
            final ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                mySubjectsCompetencesList.add(new SubjectCompetence(rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ED_GROUP), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_DESCRIPTION), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER), rs.getDate(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_I_DATE), rs.getDate(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_F_DATE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ASS_S_C_ID_TEACHER), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ASS_S_C_TITLE), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_NON_ACADEMIC_DAYS),
                        rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_POSITIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_NEGATIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_POSITIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_NEGATIVE)));
            }
        } catch (final Exception e) {
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return mySubjectsCompetencesList;
        }
    }

    /**
     *
     * @param idEdmodo Edmodo user id.
     * @param title SubjectCompetence's title to find.
     * @return All data of SubjectCompetence or NULL if any was found.
     * @throws IllegalArgumentException If any argument is not valid.
     */
    static SubjectCompetence getSubjectCompetence(final int idTeacher, final String title) throws IllegalArgumentException {
        if (idTeacher == ConstantsDB.TABLE_NULL_ID) {
            throw new IllegalArgumentException("No valid Edmodo id.");
        }
        if (title == null || title.length() > ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE_MAX) {
            throw new IllegalArgumentException("No valid title.");
        }
        SubjectCompetence subjectCompetence = null;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            ps = con.prepareStatement(new StringBuilder("SELECT * FROM ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE)
                    .append(" WHERE ").
                    append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append("=? AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append("=?")
                    .toString());
            ps.setInt(1, idTeacher);
            ps.setString(2, title);
            final ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                subjectCompetence = new SubjectCompetence(rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ED_GROUP), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_DESCRIPTION), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER), rs.getDate(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_I_DATE), rs.getDate(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_F_DATE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ASS_S_C_ID_TEACHER), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ASS_S_C_TITLE), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_NON_ACADEMIC_DAYS),
                        rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_POSITIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_NEGATIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_POSITIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_NEGATIVE));
            }
        } catch (final Exception e) {
            //e.printStackTrace();
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return subjectCompetence;
        }
    }

    static List<EvaluationPeriod> getEvaluationPeriodsSubjectCompetence(final int idTeacherSC, final String title) throws IllegalArgumentException {
        if (idTeacherSC == ConstantsDB.TABLE_NULL_ID) {
            throw new IllegalArgumentException("Invalid teacher id");
        }
        if (title == null || title.length() > ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE_MAX) {
            throw new IllegalArgumentException("Invalid title");
        }
        final List<EvaluationPeriod> myEvaluationPeriods = new ArrayList<EvaluationPeriod>();
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_NAME_EVALUATION_PERIOD)
                    .append(".* FROM ").
                    append(ConstantsDB.TABLE_NAME_EVALUATION_PERIOD)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TEACHER_ID).append("=? AND ")
                    .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TITLE).append("=?")
                    .toString());
            ps.setInt(1, idTeacherSC);
            ps.setString(2, title);
            final ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                myEvaluationPeriods.add(new EvaluationPeriod(rs.getInt(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_ID_EVALUATION_PERIOD), rs.getDate(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_INITITAL_DATE), rs.getDate(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_FINAL_DATE), rs.getInt(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_PERCENTAGE), rs.getString(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TITLE), rs.getInt(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TEACHER_ID)));
            }
        } catch (final Exception e) {
            //e.printStackTrace();
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return myEvaluationPeriods;
        }
    }

    static boolean existSubjectCompetenceTeacher(final String idEdmodo, final String title) throws IllegalArgumentException {
        if (idEdmodo == null) {
            throw new IllegalArgumentException("No valid Edmodo id.");
        }
        boolean exists = false;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append(" FROM ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE)
                    .append(',').append(ConstantsDB.VIEW_TEACHER).append(" WHERE ").
                    append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append('=')
                    .append(ConstantsDB.TABLE_TEACHER_ATT_ID_TEACHER).append(" AND ")
                    .append(ConstantsDB.TABLE_TEACHER_ATT_ID_EDMODO).append("=? AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append("=?")
                    .toString());
            ps.setString(1, idEdmodo);
            ps.setString(2, title);
            if (ps.executeQuery().next()) {
                exists = true;
            }
        } catch (final Exception e) {//e.printStackTrace();
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return exists;
        }
    }

    /**
     *
     * @param subjectCompetence
     * @return value higher than {@link Constants#NO_RESULT} if all ok, value
     * {@link Constants#NO_RESULT} otherwise.
     * @throws IllegalArgumentException
     */
    static byte deleteSubjectCompetence(final int idTeacher, final String title) throws IllegalArgumentException {
        byte result = Constants.NO_RESULT + 1;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            ps = con.prepareStatement(new StringBuilder("DELETE FROM ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE).
                    append(" WHERE ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append("= ? AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append("= ?;").toString());
            ps.setInt(1, idTeacher);
            ps.setString(2, title);
            ps.executeUpdate();
        } catch (final Exception e) {//e.printStackTrace();
            result = Constants.NO_RESULT;
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return result;
        }

    }

    /**
     *
     * @param subjectCompetence
     * @return value higher than {@link Constants#NO_RESULT} if all ok, value
     * {@link Constants#NO_RESULT} otherwise.
     */
    static byte insertSubjectCompetence(final SubjectCompetence subjectCompetence, final List<EvaluationPeriod> evaluationPeriodsList) {
        byte result = Constants.NO_RESULT + 1;

        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            // Transaction
            con.setAutoCommit(false);
            // Foreign keys
            final int idAss = subjectCompetence.getAssociatedTeacherId();
            final String titleAss = subjectCompetence.getAssociatedTitle();
            final boolean insertAssociated = (idAss >= ConstantsDB.TABLE_ID_MIN) && (titleAss != null);
            ps = con.prepareStatement(new StringBuilder("INSERT INTO ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE)
                    .append("(").append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ED_GROUP).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_I_DATE).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_F_DATE).append(',')
                    .append(insertAssociated
                    ? ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ASS_S_C_ID_TEACHER + ','
                    + ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ASS_S_C_TITLE + ','
                    : "")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_DESCRIPTION).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_NON_ACADEMIC_DAYS)
                    .append(") VALUES (?,?,?,?,?").
                    append(insertAssociated ? ",?,?" : "")
                    .append(",?,?)").toString());
            ps.setInt(1, subjectCompetence.getTeacherId());
            ps.setString(2, subjectCompetence.getTitle());
            ps.setString(3, subjectCompetence.getEdmodoGroup());
            ps.setDate(4, new java.sql.Date(subjectCompetence.getInitialDate().getTime()));
            ps.setDate(5, new java.sql.Date(subjectCompetence.getFinalDate().getTime()));
            int number = 6;
            if (insertAssociated) {
                ps.setInt(number, subjectCompetence.getAssociatedTeacherId());
                ps.setString(number + 1, subjectCompetence.getAssociatedTitle());
                number = number + 2;
            }
            ps.setString(number, subjectCompetence.getDescription());
            ps.setString(number + 1, subjectCompetence.getNonAcademicDays());
            ps.executeUpdate();
            for (final EvaluationPeriod ePeriod : evaluationPeriodsList) {
                ps = con.prepareStatement(new StringBuilder("INSERT INTO ")
                        .append(ConstantsDB.TABLE_NAME_EVALUATION_PERIOD)
                        .append("(").append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_INITITAL_DATE).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_FINAL_DATE).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_PERCENTAGE).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TEACHER_ID).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TITLE)
                        .append(") VALUES (?,?,?,?,?)").toString());
                ps.setDate(1, new java.sql.Date(ePeriod.getInitialDate().getTime()));
                ps.setDate(2, new java.sql.Date(ePeriod.getFinalDate().getTime()));
                ps.setInt(3, ePeriod.getPercentage());
                ps.setInt(4, ePeriod.getAssociattedSCTeacherId());
                ps.setString(5, ePeriod.getAssociatedSCTitle());
                ps.executeUpdate();
            }
            con.commit();
        } catch (final Exception e) {
            //e.printStackTrace();
            con.rollback();
            result = Constants.NO_RESULT;
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return result;
        }
    }

    /**
     * This method not update edmodo group, one selected, IT IS FORBIDDEN TO
     * CHANGE IT.
     *
     * @return A higher value than {@link Constants#NO_RESULT} if all ok, value
     * {@link Constants#NO_RESULT} otherwise.
     */
    static byte updateSubjectCompetence(final String previousTitle, final SubjectCompetence subjectCompetence, final List<EvaluationPeriod> evaluationPeriodsList) {
        byte result = Constants.NO_RESULT + 1;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            // Transaction
            con.setAutoCommit(false);
            // 0. Foreign keys
            final int idAss = subjectCompetence.getAssociatedTeacherId();
            final String titleAss = subjectCompetence.getAssociatedTitle();
            final boolean insertAssociated = (idAss >= ConstantsDB.TABLE_ID_MIN) && (titleAss != null);
            // 1. New Subject/Competence
            ps = con.prepareStatement(new StringBuilder("UPDATE ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE)
                    .append(" SET ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_I_DATE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_F_DATE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ASS_S_C_ID_TEACHER).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ASS_S_C_TITLE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_DESCRIPTION).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_NON_ACADEMIC_DAYS).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_POSITIVE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_POSITIVE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_NEGATIVE).append("=?,")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_NEGATIVE)
                    .append("=? WHERE ").
                    append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append("=? AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append("=?")
                    .toString());
            final String newTitle = subjectCompetence.getTitle();
            int counter = 1;
            ps.setString(counter, newTitle);
            counter++;
            ps.setDate(counter, new java.sql.Date(subjectCompetence.getInitialDate().getTime()));
            counter++;
            ps.setDate(counter, new java.sql.Date(subjectCompetence.getFinalDate().getTime()));
            counter++;
            if (insertAssociated) {
                ps.setInt(counter, subjectCompetence.getAssociatedTeacherId());
                counter++;
                ps.setString(counter, subjectCompetence.getAssociatedTitle());
            } else {
                ps.setString(counter, null);
                counter++;
                ps.setString(counter, null);
            }
            counter++;
            ps.setString(counter, subjectCompetence.getDescription());
            counter++;
            final int teacherId = subjectCompetence.getTeacherId();
            ps.setString(counter, subjectCompetence.getNonAcademicDays());
            counter++;
            ps.setInt(counter, subjectCompetence.getEachPositiveWeigth());
            counter++;
            ps.setInt(counter, subjectCompetence.getMaxPositiveWeigth());
            counter++;
            ps.setInt(counter, subjectCompetence.getEachNegativeWeigth());
            counter++;
            ps.setInt(counter, subjectCompetence.getMaxNegativeWeigth());
            counter++;
            ps.setInt(counter, teacherId);
            counter++;
            ps.setString(counter, previousTitle);
            ps.executeUpdate();

            // 2. Delete previous periods
            ps = con.prepareStatement(new StringBuilder("DELETE FROM ")
                    .append(ConstantsDB.TABLE_NAME_EVALUATION_PERIOD)
                    .append(" WHERE ").append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TITLE)
                    .append("=? AND ").append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TEACHER_ID).append("=?").toString());
            ps.setString(1, newTitle);
            ps.setInt(2, teacherId);
            ps.executeUpdate();

            // 3. Insert new periods
            for (final EvaluationPeriod ePeriod : evaluationPeriodsList) {
                ps = con.prepareStatement(new StringBuilder("INSERT INTO ")
                        .append(ConstantsDB.TABLE_NAME_EVALUATION_PERIOD)
                        .append("(").append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_INITITAL_DATE).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_FINAL_DATE).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_PERCENTAGE).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TEACHER_ID).append(',')
                        .append(ConstantsDB.TABLE_EVALUATION_PERIOD_ATT_S_C_TITLE)
                        .append(") VALUES (?,?,?,?,?)").toString());
                ps.setDate(1, new java.sql.Date(ePeriod.getInitialDate().getTime()));
                ps.setDate(2, new java.sql.Date(ePeriod.getFinalDate().getTime()));
                ps.setInt(3, ePeriod.getPercentage());
                ps.setInt(4, ePeriod.getAssociattedSCTeacherId());
                ps.setString(5, newTitle);
                ps.executeUpdate();
            }
            con.commit();
        } catch (final Exception e) {
            con.rollback();
            result = Constants.NO_RESULT;
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return result;
        }
    }

    /**
     * This method not updates edmodo group and title, one time selected, IT IS
     * FORBIDDEN TO CHANGE THEM.
     *
     * @return value higher than {@link Constants#NO_RESULT} if all ok, value
     * {@link Constants#NO_RESULT} otherwise.
     */
    static byte updateNonAcademicDaysSubjectCompetence(final String title, final String idEdmodo, final String nonAcademicDays) {
        byte result = Constants.NO_RESULT + 1;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            ps = con.prepareStatement(new StringBuilder("UPDATE ")
                    .append(ConstantsDB.TABLE_NAME_SUBJECT_COMPETENCE)
                    .append(" SET ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_NON_ACADEMIC_DAYS).append("=? WHERE ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append(" IN (SELECT ")
                    .append(ConstantsDB.TABLE_TEACHER_ATT_ID_TEACHER).append(" FROM ")
                    .append(ConstantsDB.TABLE_NAME_TEACHER)
                    .append(" WHERE ").append(ConstantsDB.TABLE_TEACHER_ATT_ID_EDMODO).append("=?) AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE)
                    .append("=?")
                    .toString());
            ps.setString(1, nonAcademicDays);
            ps.setString(2, idEdmodo);
            ps.setString(3, title);
            ps.executeUpdate();
        } catch (final Exception e) {
            //e.printStackTrace();
            result = Constants.NO_RESULT;
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return result;
        }
    }
}