package classcontrol.edmodo;

import classcontrol.dao.Persistence;
import classcontrol.entity.Student;
import classcontrol.utils.Constants;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Map.Entry;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONObject;

/**
 * Handles Edmodo calls efficiently.
 *
 * @author Cristian TG
 * @since 0.2-BETA
 * @version 0.2-BETA
 */
public final class EdmodoUtil {

    /**
     * Returns a Title-Edmodo Group Id pairs list. If user is a teacher, returns
     * only groups which he is owner.
     *
     * @param idEdmodoUser Edmodo's user token.
     * @param request Current session request.
     * @param edmodoData True if this method should get data from Edmodo, false
     * ir this method should get data from session attributes. For performance.
     * @return a Title-Edmodo group id pairs list. If use is a teacher, returns
     * only groups which he is owner.
     */
    public static ArrayList<Entry<String, String>> getGroupNamesAndEdmodId(final String idEdmodoUser, final HttpServletRequest request, final boolean edmodoData) {
        final HttpSession mSession = request.getSession();
        if (edmodoData) {
            final ArrayList<Entry<String, String>> groupsTitle = new ArrayList<Entry<String, String>>();
            try {
                final boolean isTeacher = (Byte) mSession.getAttribute(Constants.SESSION_ID_WEB_ROLE) == Constants.SESSION_ID_ROLE_TEACHER;
                final JSONArray userGroups = new JSONArray(EdmodoAPI.groupsForUser((String) mSession.getAttribute(EdmodoAPI.API_ACCESS_TOKEN_KEY), idEdmodoUser));
                final int GROUPS_LENGTH = userGroups.length();
                JSONObject mJSONObjectAux;
                JSONArray mJSONArrayAux;
                int nOwners;
                boolean find;
                int counter;
                for (int i = 0; i < GROUPS_LENGTH; i++) {
                    mJSONObjectAux = userGroups.getJSONObject(i);
                    if (!isTeacher) {
                        groupsTitle.add(new AbstractMap.SimpleEntry<String, String>(mJSONObjectAux.getString(EdmodoAPI.API_GROUP_TITLE_KEY), mJSONObjectAux.getString(EdmodoAPI.API_GROUP_ID_KEY)));
                    } else {
                        mJSONArrayAux = mJSONObjectAux.getJSONArray(EdmodoAPI.API_OWNERS_GROUP_KEY);
                        nOwners = mJSONArrayAux.length();
                        find = false;
                        counter = 0;
                        while (!find && counter < nOwners) {
                            if (idEdmodoUser.compareTo(mJSONArrayAux.optString(counter)) == 0) {
                                groupsTitle.add(new AbstractMap.SimpleEntry<String, String>(mJSONObjectAux.getString(EdmodoAPI.API_GROUP_TITLE_KEY), mJSONObjectAux.getString(EdmodoAPI.API_GROUP_ID_KEY)));
                                find = true;
                            }
                            counter++;
                        }
                    }
                }
            } catch (final Exception ex) {
            }
            mSession.setAttribute(Constants.SESSION_USER_GROUPS, groupsTitle);
            return groupsTitle;
        }
        return (ArrayList<Entry<String, String>>) mSession.getAttribute(Constants.SESSION_USER_GROUPS);
    }

    /**
     * Returns a Name-Edmodo student id pairs list.
     *
     * @param idEdmodoUser Edmodo's user token.
     * @param request Current session request.
     * @param edmodoData True if this method should get data from Edmodo, false
     * ir this method should get data from session attributes.
     * @return
     */
    public static ArrayList<Entry<String, String>> getGroupMembers(final String idEdmodoGroup, final HttpServletRequest request, final boolean edmodoData) {
        final HttpSession mSession = request.getSession();
        if (edmodoData) {
            final ArrayList<Entry<String, String>> groupMembers = new ArrayList<Entry<String, String>>();
            try {
                final JSONArray membersGroup = new JSONArray(EdmodoAPI.members((String) mSession.getAttribute(EdmodoAPI.API_ACCESS_TOKEN_KEY), new String[]{idEdmodoGroup}));
                final int MEMBERS_LENGTH = membersGroup.length();
                String fullName;
                //nombre+ConstantsForm.SEPARATOR+idEdmodo
                JSONObject mJSONObject;
                int nameLength;
                for (int i = 0; i < MEMBERS_LENGTH; i++) {
                    mJSONObject = membersGroup.getJSONObject(i);
                    // Only students
                    if (mJSONObject.getString(EdmodoAPI.API_USER_TYPE_KEY).compareTo(EdmodoAPI.API_USER_TYPE_STUDENT_VALUE) == 0) {
                        fullName = mJSONObject.getString(EdmodoAPI.API_FIRST_NAME_KEY) + ' ' + mJSONObject.optString(EdmodoAPI.API_LAST_NAME_KEY);
                        nameLength = fullName.length();
                        fullName = nameLength > Constants.MAX_NAME_NAV_BAR ? fullName.substring(0, Constants.MAX_NAME_NAV_BAR) : fullName;
                        groupMembers.add(
                                new AbstractMap.SimpleEntry<String, String>(fullName, mJSONObject.getString(EdmodoAPI.API_USER_TOKEN_KEY)));
                    }
                }
            } catch (final Exception ex) {
                //ex.printStackTrace();
            }
            mSession.setAttribute(Constants.SESSION_GROUPS_MEMBERS, groupMembers);
            return groupMembers;
        }
        return (ArrayList<Entry<String, String>>) mSession.getAttribute(Constants.SESSION_GROUPS_MEMBERS);
    }

    /**
     * Returns all group members data.
     *
     * @param idEdmodoStudent Edmodo's group token.
     * @param request Current session request.
     * @param edmodoData True if this method should get data from Edmodo, false
     * ir this method should get data from session attributes.
     * @return All group members data.
     */
    public static ArrayList<Student> getGroupStudents(final String idEdmodoGroup, final HttpServletRequest request, final boolean edmodoData) {
        // 1. Students' edmodo id list.
        // 2. Students' all datda
        return new Persistence().getStudentList(EdmodoUtil.getGroupMembers(idEdmodoGroup, request, edmodoData));
    }

    /**
     * Returns all data about a student.
     *
     * @param idEdmodoStudent Edmodo's student token.
     * @param request Current session request.
     * @return All data about a student.
     */
    public static Student getStudent(final String idEdmodoStudent, final HttpServletRequest request) {
        final JSONObject mjson;
        try {
            mjson = new JSONArray(EdmodoAPI.users((String) request.getSession().getAttribute(EdmodoAPI.API_ACCESS_TOKEN_KEY), new String[]{idEdmodoStudent})).getJSONObject(0);
            return new Student(new Persistence().getIdStudent(idEdmodoStudent), idEdmodoStudent, mjson.getString(EdmodoAPI.API_FIRST_NAME_KEY) + ' ' + mjson.optString(EdmodoAPI.API_LAST_NAME_KEY));
        } catch (final Exception ex) {
            return null;
        }
    }
}