package classcontrol.controller;

import classcontrol.dao.ConstantsForm;
import classcontrol.dao.Persistence;
import classcontrol.edmodo.EdmodoAPI;
import classcontrol.edmodo.EdmodoUtil;
import classcontrol.entity.Assignment;
import classcontrol.entity.EvaluationPeriodAssignment;
import classcontrol.entity.Student;
import classcontrol.entity.SubjectCompetence;
import classcontrol.language.LanguageFactory;
import classcontrol.utils.Constants;
import classcontrol.utils.CurrentLanguage;
import classcontrol.utils.Login;
import classcontrol.utils.Text;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.FontFactory;
import com.lowagie.text.PageSize;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.html.simpleparser.HTMLWorker;
import com.lowagie.text.pdf.FontSelector;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfPageEventHelper;
import com.lowagie.text.pdf.PdfWriter;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.AbstractMap;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * Generates a pdf report in function to specified options.
 *
 * @author Cristian TG
 * @since 0.1-ALPHA
 * @version 1.0-INITIAL
 */
public final class PdfUtil {

    private HttpServletRequest request;
    final byte MARGIN = 40;

    /**
     * Generates a pdf report with specified options in params.
     *
     * @param req Request.
     * @param scList Subject competences list to get reports.
     * @param typeList List of report types:
     * {@link ConstantsForm#GENERATE_REPORTS_OPTIONS}
     * @param orientation Report page orientation
     * {@link ConstantsForm#GENERATE_REPORTS_ORIENTATION}
     * @param startDate Start date to get report info.
     * @param endDate Enda date to get report info.
     * @param possibleIdStudent If it is specified report will only contains
     * this student's dat.a
     * @param possibleNameStudent Name of
     * @return A pdf in {@link ByteArrayOutputStream} format with specified
     * options in params.
     * @throws IOException If something bad occurs.
     */
    public ByteArrayOutputStream generatePDF(final HttpServletRequest req, final List<SubjectCompetence> scList, final List<Byte> typeList, final byte orientation, final Date startDate, final Date endDate, final int possibleIdStudent, final String possibleNameStudent) throws IOException {
        request = req;
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        final HttpSession session = request.getSession();
        // In future versions, it could be good all encoding formats supported:
        Integer languageSessionID = (Integer) session.
                getAttribute(Constants.SESSION_ID_LANGUAGE);
        final int preLanguageSessionID = languageSessionID != null ? languageSessionID : LanguageFactory.ID_LANGUAGE_DEFAULT;
        languageSessionID = !Arrays.asList(LanguageFactory.ID_LANGUAGES_SUPPORT_UTF8).contains(languageSessionID)
                ? LanguageFactory.ID_LANGUAGE_ENGLISH : languageSessionID;
        session.setAttribute(Constants.SESSION_ID_LANGUAGE, languageSessionID);
        try {
            // 1. Report type
            final boolean typeAttendReport = typeList.contains(ConstantsForm.GENERATE_REPORTS_OPTIONS[ConstantsForm.GENERATE_REPORTS_ATTEND_OPTION_INDEX]);
            final boolean typeBehaviourAttitudeReport = typeList.contains(ConstantsForm.GENERATE_REPORTS_OPTIONS[ConstantsForm.GENERATE_REPORTS_BEHAVIOUR_ATTITUDE_OPTION_INDEX]);
            final boolean typeAssignmentsReport = typeList.contains(ConstantsForm.GENERATE_REPORTS_OPTIONS[ConstantsForm.GENERATE_REPORTS_ASSIGNMENTS_OPTION_INDEX]);
            Document document;
            if (orientation == ConstantsForm.GENERATE_REPORTS_ORIENTATION[ConstantsForm.GENERATE_REPORTS_ORIENTATION_HORIZONTAL_INDEX]) {
                document = new Document(PageSize.LETTER.rotate(), MARGIN, MARGIN, MARGIN, MARGIN + 20);
            } else {
                document = new Document(PageSize.A4, MARGIN, MARGIN, MARGIN, MARGIN + 20);
            }

            PdfWriter.getInstance(document, baos).setPageEvent(new FooterTable(document));
            document.open();
            document.addTitle("Class Control");

            final String attendReportTitle = CurrentLanguage.getString(Text.S_C_ATTEND, request);
            final String behaviourAttitudeReportTitle = CurrentLanguage.getString(Text.S_C_BEHAVIOUR_ATTITUDE, request);
            final String assignmentsReportTitle = CurrentLanguage.getString(Text.S_C_ASSIGNMENTS, request);
            document.addSubject((typeAttendReport ? attendReportTitle + ' ' : "")
                    + (typeBehaviourAttitudeReport ? behaviourAttitudeReportTitle + ' ' : "")
                    + (typeAssignmentsReport ? assignmentsReportTitle : ""));
            document.addAuthor("Cristian TG");
            document.addCreator("Class Control");
            document.addProducer();
            document.addCreationDate();

            final HTMLWorker htmlWorker = new HTMLWorker(document);
            String associatedTo;
            final String startToEnd = ConstantsForm.DATE_SIMPLE_DATE_FORMAT.format(startDate) + ' ' + CurrentLanguage.getString(Text.TO, request) + ' ' + ConstantsForm.DATE_SIMPLE_DATE_FORMAT.format(endDate);
            String title;
            final String associatedTitle = CurrentLanguage.getString(Text.CSC_TEACHER_ASSOCIATED, request);
            final AssignmentUtil mAssignmentUtil = new AssignmentUtil(request);
            List<EvaluationPeriodAssignment> mAssignmentsEvalationPeriod;
            List<Assignment> mAssignments;
            Date dateAux;
            SubjectCompetence scAux;
            final String maxCalTitle = CurrentLanguage.getString(Text.CREATE_ASSIGNMENT_MAX_QUALIFICATION, request);

            final FontSelector selector = new FontSelector();
            selector.addFont(FontFactory.getFont(FontFactory.TIMES_ROMAN, 24));
            final Paragraph preface = new Paragraph(selector.process(Login.getNameLastFull(request)) + (possibleNameStudent != null ? " - " + possibleNameStudent : ""));
            preface.setAlignment(Element.ALIGN_CENTER);
            document.add(preface);
            document.add(Chunk.NEWLINE);
            final FontSelector dateSelector = new FontSelector();
            dateSelector.addFont(FontFactory.getFont(FontFactory.TIMES_ROMAN, 12));
            final Paragraph mDate = new Paragraph(dateSelector.process(ConstantsForm.DATE_SIMPLE_DATE_FORMAT.format(Calendar.getInstance().getTime())));
            mDate.setAlignment(Element.ALIGN_CENTER);
            document.add(mDate);
            int associatedTeacherId;
            int scTeacherId;
            // 2. For each Subject Competence
            final Set<Map.Entry<String, Integer>> pairList = new HashSet<Map.Entry<String, Integer>>();
            final byte role = (Byte) session.getAttribute(Constants.SESSION_ID_WEB_ROLE);
            final boolean isTeacher = role == Constants.SESSION_ID_ROLE_TEACHER;
            List<Student> subjectCompetenceMembers;
            String gradesGroupAux;
            final String accessToken = (String) session.getAttribute(EdmodoAPI.API_ACCESS_TOKEN_KEY);
            final Persistence mPersistence = new Persistence();
            String assignmentsUserAux;
            String edmodoGroup;

            for (final SubjectCompetence sc : scList) {
                edmodoGroup = sc.getEdmodoGroup();
                subjectCompetenceMembers = EdmodoUtil.getGroupStudents(edmodoGroup, request, false);
                title = sc.getTitle();
                scTeacherId = sc.getTeacherId();
                assignmentsUserAux = EdmodoAPI.assignmentsByAppForUser(accessToken, mPersistence.getIdEdmodoTeacher(scTeacherId));
                // 1. Types:
                if (typeAttendReport) {
                    htmlWorker.parse(new StringReader("<br/><h2>" + attendReportTitle + ": " + title + "</h2><br/><h4>" + startToEnd + "</h4><br/><br/>"));
                    htmlWorker.parse(new StringReader(TableHtml.getAttendReport(sc, request, false, startDate, endDate, possibleIdStudent, subjectCompetenceMembers).toString()));
                    document.newPage();
                }
                if (typeBehaviourAttitudeReport) {
                    htmlWorker.parse(new StringReader("<br/><h2>" + behaviourAttitudeReportTitle + ": " + title + "</h2><br/><h4>" + startToEnd + "</h4><br/><br/>"));
                    htmlWorker.parse(new StringReader(TableHtml.getBehaviourAttitudeReport(sc, request, false, startDate, endDate, possibleIdStudent, subjectCompetenceMembers).toString()));
                    document.newPage();
                }
                if (typeAssignmentsReport && pairList.add(new AbstractMap.SimpleEntry<String, Integer>(title, scTeacherId))) {
                    htmlWorker.parse(new StringReader("<br/><h2>" + assignmentsReportTitle + ": " + title + "</h2><br/><h4>" + startToEnd + "</h4><br/>"));
                    gradesGroupAux = EdmodoAPI.gradesSetByAppForGroup(accessToken, edmodoGroup);
                    htmlWorker.parse(new StringReader("<br/>" + TableHtml.getEvaluationReport(sc, request, new SubjectCompetenceUtil(request), false, startDate, endDate, possibleIdStudent, subjectCompetenceMembers, gradesGroupAux, assignmentsUserAux, htmlWorker).toString()));

                    mAssignmentsEvalationPeriod = mAssignmentUtil.getAllAssignmentsEvaluationperiod(edmodoGroup, scTeacherId, title, false, gradesGroupAux, assignmentsUserAux);
                    document.newPage();
                    for (final EvaluationPeriodAssignment current : mAssignmentsEvalationPeriod) {
                        mAssignments = current.getMyAssignmentsList();
                        for (final Assignment assignment : mAssignments) {
                            dateAux = assignment.getDate();
                            if (!dateAux.before(startDate) && !dateAux.after(endDate)) {
                                htmlWorker.parse(new StringReader("<h3>" + title + " - " + assignment.getTitle() + ": " + ConstantsForm.DATE_SIMPLE_DATE_FORMAT.format(dateAux) + ". " + maxCalTitle + ": " + assignment.getMaxGrade() + ".</h3><br/>"));
                                htmlWorker.parse(new StringReader("<br/>" + TableHtml.getAssignmentqualifications(assignment.getIdAssignment(), sc, request, false, possibleIdStudent, role, subjectCompetenceMembers, gradesGroupAux, assignmentsUserAux, htmlWorker).toString()));
                                if (isTeacher) {
                                    document.newPage();
                                } else {
                                    document.add(Chunk.NEWLINE);
                                    document.add(Chunk.NEWLINE);
                                }
                            }
                        }
                    }

                    associatedTo = sc.getAssociatedTitle();
                    associatedTeacherId = sc.getAssociatedTeacherId();
                    if (associatedTo != null && pairList.add(new AbstractMap.SimpleEntry<String, Integer>(associatedTo, associatedTeacherId))) {
                        document.newPage();
                        htmlWorker.parse(new StringReader("<h2>" + title + " - " + associatedTitle + ": " + associatedTo + "</h3><br/><h4>" + startToEnd + "</h2><br/>"));
                        scAux = mPersistence.getSubjectCompetence(associatedTeacherId, associatedTo);
                        edmodoGroup = scAux.getEdmodoGroup();
                        gradesGroupAux = EdmodoAPI.gradesSetByAppForGroup(accessToken, edmodoGroup);
                        htmlWorker.parse(new StringReader("<br/>" + TableHtml.getEvaluationReport(scAux, request, new SubjectCompetenceUtil(request), false, startDate, endDate, possibleIdStudent, subjectCompetenceMembers, gradesGroupAux, assignmentsUserAux, htmlWorker).toString()));
                        document.newPage();
                        mAssignmentsEvalationPeriod = mAssignmentUtil.getAllAssignmentsEvaluationperiod(edmodoGroup, associatedTeacherId, scAux.getTitle(), false, gradesGroupAux, assignmentsUserAux);

                        for (final EvaluationPeriodAssignment current : mAssignmentsEvalationPeriod) {
                            mAssignments = current.getMyAssignmentsList();
                            for (final Assignment assignment : mAssignments) {
                                dateAux = assignment.getDate();
                                if (!dateAux.before(startDate) && !dateAux.after(endDate)) {
                                    htmlWorker.parse(new StringReader("<h3>" + associatedTo + " - " + assignment.getTitle() + ": " + ConstantsForm.DATE_SIMPLE_DATE_FORMAT.format(dateAux) + ". " + maxCalTitle + ": " + assignment.getMaxGrade() + ".</h3><br/>"));
                                    htmlWorker.parse(new StringReader("<br/>" + TableHtml.getAssignmentqualifications(assignment.getIdAssignment(), scAux, request, false, possibleIdStudent, role, subjectCompetenceMembers, gradesGroupAux, assignmentsUserAux, htmlWorker).toString()));
                                    if (isTeacher) {
                                        document.newPage();
                                    } else {
                                        document.add(Chunk.NEWLINE);
                                        document.add(Chunk.NEWLINE);
                                    }
                                }
                            }
                        }
                    }
                    document.newPage();
                }
            }
            document.close();

        } catch (final DocumentException e) {
            throw new IOException(e.getMessage());
        } finally {
            // In future versions, it could be good all encoding formats supported:
            session.setAttribute(Constants.SESSION_ID_LANGUAGE, preLanguageSessionID);
            return baos;
        }
    }

    /**
     * Footer used in pdf report.
     */
    final class FooterTable extends PdfPageEventHelper {

        /**
         * PDF document report.
         */
        Document document;
        /**
         * Cell by default in footer.
         */
        final PdfPCell cellLeft = new PdfPCell(new Phrase("© Class Control - " + CurrentLanguage.getString(Text.FOOTER_CREATED_BY, request) + " Cristian TG"));

        /**
         * Basic constructor
         *
         * @param document Document to put this footer.
         */
        public FooterTable(final Document document) {
            this.document = document;
        }

        @Override
        public void onEndPage(final PdfWriter writer, final Document document) {
            final PdfPTable table = new PdfPTable(2);
            table.setTotalWidth(document.getPageSize().getWidth() - MARGIN - MARGIN);
            cellLeft.setHorizontalAlignment(Element.ALIGN_LEFT);
            cellLeft.setBorder(Rectangle.NO_BORDER);
            table.addCell(cellLeft);
            final PdfPCell cellRight = new PdfPCell(new Phrase(String.format(CurrentLanguage.getString(Text.GENERATE_REPORTS_PAGE_TEXT, request) + " %d", writer.getPageNumber())));
            cellRight.setHorizontalAlignment(Element.ALIGN_RIGHT);
            cellRight.setBorder(Rectangle.NO_BORDER);
            table.addCell(cellRight);
            table.writeSelectedRows(0, -1, 36, 64, writer.getDirectContent());
        }
    }
}