package classcontrol.dao;

import classcontrol.entity.Qualification;
import classcontrol.entity.EvaluableItemStudent;
import classcontrol.entity.StudentQualificationReport;
import classcontrol.utils.Constants;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.List;

/**
 * Handles all qualifications' database connections.
 *
 * @author Cristian TG
 * @since 0.1-ALPHA
 * @version 1.0-INITIAL
 */
final class QualificationDAO {

    /**
     * Updates all subject competence assignment's qualifications-evaluable item
     * students.
     *
     * @param idAssignment Database assignment's id.
     * @param mStudentQualifications Student-Qualification pair list to
     * update-insert in database.
     * @return {@link Constants#NO_RESULT} if transaction coul not be performed
     * or an integer greater than {@link Constants#NO_RESULT} oherwise.
     */
    static byte updateQualificationsAssignment(final int idAssignment, final List<StudentQualificationReport> mStudentQualifications) throws IllegalArgumentException {
        if (idAssignment < ConstantsDB.TABLE_ID_MIN) {
            throw new IllegalArgumentException("Invalid idAssignment");
        }
        if (mStudentQualifications == null) {
            throw new IllegalArgumentException("Invalid mStudentQualifications");
        }
        byte result = Constants.NO_RESULT + 1;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps, qs;

        try {
            // Transaction
            con.setAutoCommit(false);
            // 1. Delete all from qualification table
            ps = con.prepareStatement(new StringBuilder("DELETE FROM ")
                    .append(ConstantsDB.TABLE_NAME_QUALIFICATION)
                    .append(" WHERE ").append(ConstantsDB.TABLE_QUALIFICATION_ATT_ASSIGNMENT_ID)
                    .append("=?").toString());
            ps.setInt(1, idAssignment);
            ps.executeUpdate();

            // 2. Delete all from EvaluableItemStudent
            ps = con.prepareStatement(new StringBuilder("DELETE FROM ")
                    .append(ConstantsDB.TABLE_NAME_EVALUABLE_ITEM_STUDENT)
                    .append(" WHERE ").append(ConstantsDB.TABLE_EVALUABLE_ITEM_STUDENT_ATT_EVALUABLE_ITEM_ASSIGNMENT_ID)
                    .append("=?").toString());
            ps.setInt(1, idAssignment);
            ps.executeUpdate();

            // 3. Insert qualifications and EvaluableItemStudent
            ps = con.prepareStatement(new StringBuilder("INSERT INTO ")
                    .append(ConstantsDB.TABLE_NAME_QUALIFICATION)
                    .append("(").append(ConstantsDB.TABLE_QUALIFICATION_ATT_ASSIGNMENT_ID).append(',')
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_STUDENT_ID).append(',')
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_BONUS_PENALIZATION).append(',')
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_REASON).append(',')
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_OPTIONAL_COMMENT).append(',')
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_VISIBLE_STUDENT).append(',')
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_VISIBLE_PARENT)
                    .append(") VALUES (?,?,?,?,?,?,?)").toString());
            qs = con.prepareStatement(new StringBuilder("INSERT INTO ")
                    .append(ConstantsDB.TABLE_NAME_EVALUABLE_ITEM_STUDENT)
                    .append("(").append(ConstantsDB.TABLE_EVALUABLE_ITEM_STUDENT_ATT_STUDENT_ID).append(',')
                    .append(ConstantsDB.TABLE_EVALUABLE_ITEM_STUDENT_ATT_SCORE).append(',')
                    .append(ConstantsDB.TABLE_EVALUABLE_ITEM_STUDENT_ATT_EVALUABLE_ITEM_ASSIGNMENT_ID).append(',')
                    .append(ConstantsDB.TABLE_EVALUABLE_ITEM_STUDENT_ATT_EVALUABLE_ITEM_NAME)
                    .append(") VALUES (?,?,?,?)").toString());
            Qualification cal;
            List<EvaluableItemStudent> mList;
            int counter;
            for (final StudentQualificationReport current : mStudentQualifications) {
                cal = current.getQualification();
                // 1. Insert qualification
                counter = 1;
                ps.setInt(counter, idAssignment);
                counter++;
                ps.setInt(counter, current.getStudent().getIdStudent());
                counter++;
                ps.setDouble(counter, cal.getBonusPenalization());
                counter++;
                ps.setString(counter, cal.getReasonBonusPenalization());
                counter++;
                ps.setString(counter, cal.getOptionalComment());
                counter++;
                ps.setByte(counter, cal.isVisibleStudent());
                counter++;
                ps.setByte(counter, cal.isVisibleParent());
                ps.executeUpdate();
                mList = current.getMyEvaluableItemQualification();
                for (final EvaluableItemStudent evaluableItemStudent : mList) {
                    counter = 1;
                    qs.setInt(counter, evaluableItemStudent.getIdStudent());
                    counter++;
                    qs.setDouble(counter, evaluableItemStudent.getScore());
                    counter++;
                    qs.setInt(counter, idAssignment);
                    counter++;
                    qs.setString(4, evaluableItemStudent.getName());
                    qs.executeUpdate();
                }
            }
            con.commit();
        } catch (final Exception ex) {
            con.rollback();
            //ex.printStackTrace();
            result = Constants.NO_RESULT;
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return result;
        }
    }
}