package classcontrol.dao;

import classcontrol.entity.Student;
import classcontrol.entity.SubjectCompetence;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import classcontrol.utils.Constants;
import java.util.AbstractMap.SimpleEntry;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * Handles all studen's database connections.
 *
 * @author Cristian TG
 * @since 0.1-ALPHA
 * @version 1.0-INITIAL
 */
final class StudentDAO {

    /**
     * Returns a specified student's id.
     *
     * @param idEdmodo Edmodo token for this student.
     * @return {@link Constants#NO_RESULT} if user not exists in Class Control
     * database. database. Number greater than {@link Constants#NO_RESULT}
     * otherwise.
     * @throws IllegalArgumentException If argunt is null.
     */
    static int getIdStudent(final String idEdmodo) throws IllegalArgumentException {
        if (idEdmodo == null) {
            throw new IllegalArgumentException("No valid Edmodo id.");
        }
        int studentId = Constants.NO_RESULT;
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        try {
            final PreparedStatement ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT).append(" FROM ")
                    .append(ConstantsDB.VIEW_STUDENT)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append(
                    "=?").toString());
            ps.setString(1, idEdmodo);
            final ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                studentId = rs.getInt(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT);
            }
        } catch (final Exception e) {
            //e.printStackTrace();
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return studentId;
        }
    }

    /**
     * Get student's Class Control application involved data.
     *
     * @param idEdmodoStudent Student to get data from.
     * @param edmodoGroupsId Filter results to student's Edmodo groups id that
     * belongs to. If it is null or empty all results are returned.
     * @return A list of subject competences that student has some data.
     */
    static List<SubjectCompetence> getInvolvedDataStudent(final String idEdmodoStudent, final ArrayList<String> edmodoGroupsId) {
        final List<SubjectCompetence> mSubjectsCompetences = new ArrayList<SubjectCompetence>();
        if (idEdmodoStudent == null) {
            return mSubjectsCompetences;
        }
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        PreparedStatement ps;
        try {
            final Set<Entry<String, Integer>> pairList = new HashSet<Entry<String, Integer>>();

            // 1. Look for student's absences
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_ABSENCE_ATT_S_C_TEACHER_ID).append(',')
                    .append(ConstantsDB.TABLE_ABSENCE_ATT_S_C_TITLE)
                    .append(" FROM ")
                    .append(ConstantsDB.VIEW_ABSENCE).append(',').append(ConstantsDB.TABLE_NAME_STUDENT)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_ABSENCE_ATT_STUDENT_ID).append('=').
                    append(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT).append(" AND ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append(" =?").toString());
            ps.setString(1, idEdmodoStudent);
            ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                pairList.add(new SimpleEntry<String, Integer>(rs.getString(ConstantsDB.TABLE_ABSENCE_ATT_S_C_TITLE), rs.getInt(ConstantsDB.TABLE_ABSENCE_ATT_S_C_TEACHER_ID)));
            }
            // 2. Look for student's behaviour attitude aspects
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_ASPECT_STUDENT_ATT_S_C_TEACHER_ID).append(',')
                    .append(ConstantsDB.TABLE_ASPECT_STUDENT_ATT_S_C_TITLE)
                    .append(" FROM ")
                    .append(ConstantsDB.VIEW_ASPECT_STUDENT).append(',').append(ConstantsDB.TABLE_NAME_STUDENT)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_ASPECT_STUDENT_ATT_STUDENT_ID).append('=').
                    append(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT).append(" AND ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append(" =?").toString());
            ps.setString(1, idEdmodoStudent);
            rs = ps.executeQuery();
            while (rs.next()) {
                pairList.add(new SimpleEntry<String, Integer>(rs.getString(ConstantsDB.TABLE_ASPECT_STUDENT_ATT_S_C_TITLE), rs.getInt(ConstantsDB.TABLE_ASPECT_STUDENT_ATT_S_C_TEACHER_ID)));
            }

            // 3. Look for applied student's evaluate items
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE)
                    .append(" FROM ")
                    .append(ConstantsDB.VIEW_EVALUABLE_ITEM_STUDENT).append(',').append(ConstantsDB.TABLE_NAME_STUDENT)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_EVALUABLE_ITEM_STUDENT_ATT_STUDENT_ID).append('=').
                    append(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT).append(" AND ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append(" =?").toString());
            ps.setString(1, idEdmodoStudent);
            rs = ps.executeQuery();
            while (rs.next()) {
                pairList.add(new SimpleEntry<String, Integer>(rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER)));
            }

            // 4. Look for student's qualifications
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append(',')
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE)
                    .append(" FROM ")
                    .append(ConstantsDB.VIEW_QUALIFICATION).append(',').append(ConstantsDB.TABLE_NAME_STUDENT)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_QUALIFICATION_ATT_STUDENT_ID).append('=').
                    append(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT).append(" AND ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append(" =?").toString());
            ps.setString(1, idEdmodoStudent);
            rs = ps.executeQuery();
            while (rs.next()) {
                pairList.add(new SimpleEntry<String, Integer>(rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER)));
            }

            // 5. Look for student's bonus penalizations
            ps = con.prepareStatement(new StringBuilder("SELECT ")
                    .append(ConstantsDB.TABLE_BONUS_PENALIZATION_ATT_S_C_TEACHER_ID).append(',')
                    .append(ConstantsDB.TABLE_BONUS_PENALIZATION_ATT_S_C_TITLE)
                    .append(" FROM ")
                    .append(ConstantsDB.VIEW_BONUS_PENALIZATION).append(',').append(ConstantsDB.TABLE_NAME_STUDENT)
                    .append(" WHERE ")
                    .append(ConstantsDB.TABLE_BONUS_PENALIZATION_ATT_STUDENT_ID).append('=').
                    append(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT).append(" AND ")
                    .append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append(" =?").toString());
            ps.setString(1, idEdmodoStudent);
            rs = ps.executeQuery();
            while (rs.next()) {
                pairList.add(new SimpleEntry<String, Integer>(rs.getString(ConstantsDB.TABLE_BONUS_PENALIZATION_ATT_S_C_TITLE), rs.getInt(ConstantsDB.TABLE_BONUS_PENALIZATION_ATT_S_C_TEACHER_ID)));
            }

            ps = con.prepareStatement(new StringBuilder("SELECT * FROM ")
                    .append(ConstantsDB.VIEW_SUBJECT_COMPETENCE)
                    .append(" WHERE ").append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER).append("=? AND ")
                    .append(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE).append("=?").toString());
            int totalSC = 0;
            for (final Entry<String, Integer> current : pairList) {
                ps.setInt(1, current.getValue());
                ps.setString(2, current.getKey());
                rs = ps.executeQuery();
                if (rs.next()) {
                    mSubjectsCompetences.add(new SubjectCompetence(rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_TITLE), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ED_GROUP), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_DESCRIPTION), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_TEACHER), rs.getDate(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_I_DATE), rs.getDate(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_F_DATE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ID_ASS_S_C_ID_TEACHER), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_ASS_S_C_TITLE), rs.getString(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_NON_ACADEMIC_DAYS),
                            rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_POSITIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_EACH_NEGATIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_POSITIVE), rs.getInt(ConstantsDB.TABLE_SUBJECT_COMPETENCE_ATT_MAX_NEGATIVE)));
                    totalSC++;
                }
            }
            // Filter results
            if (edmodoGroupsId != null) {
                final ArrayList<Integer> deleteIndexes = new ArrayList<Integer>();
                int index;
                for (int i = 0; i < totalSC; i++) {
                    index = edmodoGroupsId.indexOf(mSubjectsCompetences.get(i).getEdmodoGroup());
                    if (index == -1) {
                        deleteIndexes.add(0, i);
                    }
                }
                for (final Integer current : deleteIndexes) {
                    mSubjectsCompetences.remove((int) current);
                }
            }
        } catch (final Exception e) {
            //e.printStackTrace();
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            return mSubjectsCompetences;
        }
    }

    /**
     * Returns a sorted by full name students list of an Edmodo Group.
     *
     * @param edmodoIdNameStudent Pair of Edmodo Id - namestudents list.
     * @return A sorted by full name students list of an Edmodo Group.
     */
    static ArrayList<Student> getStudentList(final ArrayList<Map.Entry<String, String>> edmodoIdNameStudent) {
        final ArrayList<Student> mStudentList = new ArrayList<Student>();
        if (edmodoIdNameStudent == null) {
            return mStudentList;
        }

        final StringBuilder query = new StringBuilder("SELECT * FROM ")
                .append(ConstantsDB.VIEW_STUDENT)
                .append(" WHERE ");
        final int USERS_SIZE = edmodoIdNameStudent.size();
        final int USERS_SIZE_2 = USERS_SIZE - 2;
        for (int i = USERS_SIZE_2; i >= 0; i--) {
            query.append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append("=? OR ");
        }
        query.append(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO).append("=? ");
        final ConnectionPool pool = ConnectionPool.getInstance();
        final Connection con = pool.getConnection();
        try {
            final PreparedStatement ps = con.prepareStatement(query.toString());
            int counter = 1;
            for (final Map.Entry<String, String> current : edmodoIdNameStudent) {
                ps.setString(counter, current.getKey());//Edmodo student id
                counter++;
            }
            final ResultSet rs = ps.executeQuery();
            boolean find;
            int count;
            String idEdmodoAux;
            String valueEdmodoId;
            Map.Entry<String, String> entryAux;
            while (rs.next()) {
                idEdmodoAux = rs.getString(ConstantsDB.TABLE_STUDENT_ATT_ID_EDMODO);
                find = false;
                count = 0;
                while (!find && count < USERS_SIZE) {
                    entryAux = edmodoIdNameStudent.get(count);
                    valueEdmodoId = entryAux.getKey();
                    if (valueEdmodoId.compareTo(idEdmodoAux) == 0) {
                        mStudentList.add(new Student(rs.getInt(ConstantsDB.TABLE_STUDENT_ATT_ID_STUDENT), idEdmodoAux,
                                entryAux.getValue()));
                        find = true;
                    }
                    count++;
                }
            }
        } catch (final Exception e) {
        } finally {
            if (con != null) {
                pool.freeConnection(con);
            }
            Collections.sort(mStudentList);
            return mStudentList;
        }
    }
}