package classcontrol.language;

import classcontrol.utils.Text;
import java.util.ArrayList;

/**
 *
 * Provides specified language data.
 *
 * ATTENTION:If any language java class names changes, class's methods must be
 * changed too.
 *
 * @author Cristian TG
 * @since 0.1-ALPHA
 * @version 1.0-INITIAL
 */
public final class LanguageFactory {

    /**
     * English language identificator.
     */
    public final static int ID_LANGUAGE_ENGLISH = 0;
    /**
     * Spanish language identificator.
     */
    public final static int ID_LANGUAGE_SPANISH = 1;
    /**
     * Greek language identificator.
     */
    public final static int ID_LANGUAGE_GREEK = 2;
    /**
     * Croatian language identificator.
     */
    public final static int ID_LANGUAGE_CROATIAN = 3;
    /**
     * Russian language identificator.
     */
    public final static int ID_LANGUAGE_RUSSIAN = 4;
    /**
     * Slovenian language identificator.
     */
    public final static int ID_LANGUAGE_SLOVENIAN = 5;
    /**
     * Default language identificator.
     */
    public final static int ID_LANGUAGE_DEFAULT = ID_LANGUAGE_ENGLISH;
    /**
     * Languages id that support UTF-8 encoding.
     */
    public final static Integer[] ID_LANGUAGES_SUPPORT_UTF8 = new Integer[]{ID_LANGUAGE_ENGLISH, ID_LANGUAGE_SPANISH};

    /**
     * Provides native language name.
     *
     * @param idLanguage Language idetificator.
     * @return Language name.
     */
    public static String getLanguageById(final int idLanguage) {
        final StringBuilder string = new StringBuilder("");
        final String ENGLISH_NAME = new EnglishLanguage().getLanguageName();
        switch (idLanguage) {
            case ID_LANGUAGE_ENGLISH:
                string.append(ENGLISH_NAME);
                break;
            case ID_LANGUAGE_SPANISH:
                string.append(new SpanishLanguage().getLanguageName());
                break;
            case ID_LANGUAGE_GREEK:
                string.append(new GreekLanguage().getLanguageName());
                break;
            case ID_LANGUAGE_CROATIAN:
                string.append(new CroatianLanguage().getLanguageName());
                break;
            case ID_LANGUAGE_RUSSIAN:
                string.append(new RussianLanguage().getLanguageName());
                break;
            case ID_LANGUAGE_SLOVENIAN:
                string.append(new SlovenianLanguage().getLanguageName());
                break;
            default:
                string.append(ENGLISH_NAME);
                break;
        }
        return string.toString();
    }

    /**
     * Retruns a Class Control language identificator or
     * {@link LanguageFactory#ID_LANGUAGE_DEFAULT} if not matches.
     *
     * @param code
     * @returna Class Control language identificator or
     * {@link LanguageFactory#ID_LANGUAGE_DEFAULT} if not matches.
     */
    public static byte getLanguageIdByCode(final String code) {
        if (code != null) {
            if (code.toLowerCase().compareTo(new EnglishLanguage().getString(Text.HTML_LANG).toLowerCase()) == 0) {
                return ID_LANGUAGE_ENGLISH;
            }
            if (code.toLowerCase().compareTo(new SpanishLanguage().getString(Text.HTML_LANG).toLowerCase()) == 0) {
                return ID_LANGUAGE_SPANISH;
            }
            if (code.toLowerCase().compareTo(new GreekLanguage().getString(Text.HTML_LANG).toLowerCase()) == 0) {
                return ID_LANGUAGE_GREEK;
            }
            if (code.toLowerCase().compareTo(new CroatianLanguage().getString(Text.HTML_LANG).toLowerCase()) == 0) {
                return ID_LANGUAGE_CROATIAN;
            }
            if (code.toLowerCase().compareTo(new RussianLanguage().getString(Text.HTML_LANG).toLowerCase()) == 0) {
                return ID_LANGUAGE_RUSSIAN;
            }
            if (code.toLowerCase().compareTo(new SlovenianLanguage().getString(Text.HTML_LANG).toLowerCase()) == 0) {
                return ID_LANGUAGE_SLOVENIAN;
            }
        }
        return ID_LANGUAGE_DEFAULT;
    }

    /**
     * Provides language Java fully qualified class name.
     *
     * @param idLanguage Language identificator
     * @return Languages class name.
     */
    public static String getLanguageClassById(final int idLanguage) {
        final StringBuilder string = new StringBuilder("classcontrol.language.");
        final String ENGLISH_NAME = "English";
        switch (idLanguage) {
            case ID_LANGUAGE_ENGLISH:
                string.append(ENGLISH_NAME);
                break;
            case ID_LANGUAGE_SPANISH:
                string.append("Spanish");
                break;
            case ID_LANGUAGE_GREEK:
                string.append("Greek");
                break;
            case ID_LANGUAGE_CROATIAN:
                string.append("Croatian");
                break;
            case ID_LANGUAGE_RUSSIAN:
                string.append("Russian");
                break;
            case ID_LANGUAGE_SLOVENIAN:
                string.append("Slovenian");
                break;
            default:
                string.append(ENGLISH_NAME);
                break;
        }
        return string.append("Language").toString();
    }

    /**
     * Provides all available language names.
     *
     * @return List of all available language names.
     */
    public static ArrayList<Language> getAllLanguages() {
        final ArrayList<Language> languagesList = new ArrayList<Language>();
        languagesList.add(new Language(new EnglishLanguage().getLanguageName(), ID_LANGUAGE_ENGLISH));
        languagesList.add(new Language(new SpanishLanguage().getLanguageName(), ID_LANGUAGE_SPANISH));
        languagesList.add(new Language(new GreekLanguage().getLanguageName(), ID_LANGUAGE_GREEK));
        languagesList.add(new Language(new CroatianLanguage().getLanguageName(), ID_LANGUAGE_CROATIAN));
        languagesList.add(new Language(new RussianLanguage().getLanguageName(), ID_LANGUAGE_RUSSIAN));
        languagesList.add(new Language(new SlovenianLanguage().getLanguageName(), ID_LANGUAGE_SLOVENIAN));
        return languagesList;
    }
}